<?php

/**
 * Class ControllerExtensionPaymentImoje
 */
class ControllerExtensionPaymentImoje extends Controller
{
	private $error = [];

	/**
	 * view admin panel
	 */
	public function index()
	{
		$this->language->load('extension/payment/imoje');
		$this->document->setTitle($this->language->get('heading_title'));
		$this->document->addScript('view/javascript/select2/js/select2.min.js');
		$this->document->addStyle('view/javascript/select2/css/select2.css');
		$this->load->model('setting/setting');

		$request = $this->request->post;

		if(($this->request->server['REQUEST_METHOD'] == 'POST')
			&& $this->validate($request)
		) {

			$settings = array_merge($this->model_setting_setting->getSetting('payment_imoje'), $request);
			$this->model_setting_setting->editSetting('payment_imoje', $settings);
			$this->session->data['success'] = $this->language->get('text_success');
			$this->response->redirect($this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true));

			return;
		}

		include_once(DIR_SYSTEM . '/library/Payment-core/vendor/autoload.php');

		$data = [

			'imoje_debug_mode' => isset($this->request->post['payment_debug_mode'])
				? $this->request->post['payment_debug_mode']
				: $this->config->get('payment_debug_mode'),

			'imoje_status' => isset($this->request->post['payment_imoje_status'])
				? $this->request->post['payment_imoje_status']
				: $this->config->get('payment_imoje_status'),

			'imoje_sandbox' => isset($this->request->post['payment_imoje_sandbox'])
				? $this->request->post['payment_imoje_sandbox']
				: $this->config->get('payment_imoje_sandbox'),

			'imoje_merchant_id' => isset($this->request->post['payment_imoje_merchant_id'])
				? $this->request->post['payment_imoje_merchant_id']
				: $this->config->get('payment_imoje_merchant_id'),

			'imoje_service_id' => isset($this->request->post['payment_imoje_service_id'])
				? $this->request->post['payment_imoje_service_id']
				: $this->config->get('payment_imoje_service_id'),

			'imoje_service_key' => isset($this->request->post['payment_imoje_service_key'])
				? $this->request->post['payment_imoje_service_key']
				: $this->config->get('payment_imoje_service_key'),

			'imoje_sort_order' => isset($this->request->post['payment_imoje_sort_order'])
				? $this->request->post['payment_imoje_sort_order']
				: $this->config->get('payment_imoje_sort_order'),

			'imoje_send_cart' => isset($this->request->post['payment_imoje_send_cart'])
				? $this->request->post['payment_imoje_send_cart']
				: $this->config->get('payment_imoje_send_cart'),

			'imoje_order_history' => isset($this->request->post['payment_imoje_order_history'])
				? $this->request->post['payment_imoje_order_history']
				: $this->config->get('payment_imoje_order_history'),

			'imoje_payment_title_default' => $this->language->get('text_imoje_payment_title_default'),
			'imoje_payment_title'         => isset($this->request->post['payment_imoje_payment_title'])
				? $this->request->post['payment_imoje_payment_title']
				: $this->config->get('payment_imoje_payment_title'),

			'imoje_all_currencies' => \Imoje\Payment\Util::getSupportedCurrencies(),
			'imoje_currencies'     => isset($this->request->post['payment_imoje_currencies'])
				? $this->request->post['payment_imoje_currencies']
				: ($this->config->get('payment_imoje_currencies')
					?: []),

			'imoje_is_keys' => $this->config->get('payment_imoje_merchant_id')
				&& $this->config->get('payment_imoje_service_id')
				&& $this->config->get('payment_imoje_service_key'),

			'error_validation' => $this->error,

			'notification_url' => HTTPS_CATALOG . 'index.php?route=extension/payment/imoje/notification',

			'confirm_url' => $this->url->link('extension/payment/imoje', 'user_token=' . $this->session->data['user_token'], true),
			'cancel_url'  => $this->url->link('extension/extension', 'user_token=' . $this->session->data['user_token'], true),

			'content_footer'      => $this->load->controller('common/footer'),
			'content_header'      => $this->load->controller('common/header'),
			'content_column_left' => $this->load->controller('common/column_left'),
			'heading_title'       => $this->language->get('text_heading_title'),

			'breadcrumbs' => [
				[
					'text'      => $this->language->get('text_home'),
					'href'      => $this->url->link('common/home', 'user_token=' . $this->session->data['user_token'], 'SSL'),
					'separator' => false,
				],

				[
					'text'      => $this->language->get('text_payment'),
					'href'      => $this->url->link('extension/extension', 'user_token=' . $this->session->data['user_token'], 'SSL'),
					'separator' => ' :: ',
				],

				[
					'text'      => $this->language->get('heading_title'),
					'href'      => $this->url->link('extension/payment/imoje', 'user_token=' . $this->session->data['user_token'], 'SSL'),
					'separator' => ' :: ',
				],
			],

		];

		$this->response->setOutput($this->load->view('extension/payment/imoje', $data));

		$imoje_settings = $this->model_setting_setting->getSetting('payment_imoje');

		if(isset($imoje_settings['payment_imoje_error_tmp'])) {
			unset($imoje_settings['payment_imoje_error_tmp']);
			$this->model_setting_setting->editSetting('payment_imoje', $imoje_settings);
		}

		if(isset($imoje_settings['payment_imoje_success_tmp'])) {
			unset($imoje_settings['payment_imoje_success_tmp']);
			$this->model_setting_setting->editSetting('payment_imoje', $imoje_settings);
		}

		if(isset($imoje_settings['payment_imoje_no_twisto'])) {
			unset($imoje_settings['payment_imoje_no_twisto']);
			$this->model_setting_setting->editSetting('payment_imoje', $imoje_settings);
		}
	}

	/**
	 * @param array $request
	 *
	 * @return bool
	 */
	protected function validate($request)
	{
		if(!$this->user->hasPermission('modify', 'extension/payment/imoje')) {
			$this->error['permission'] = true;

			return false;
		}

		include_once(DIR_SYSTEM . 'library/Payment-core/vendor/autoload.php');

		$schema = [
			'type'       => 'object',
			'properties' => [

				'payment_imoje_status'        => [
					'type' => 'string',
					"enum" => [
						'0',
						'1',
					],
				],
				'payment_imoje_debug_mode'    => [
					'type' => 'string',
					"enum" => [
						'0',
						'1',
					],
				],
				'payment_imoje_sandbox'       => [
					'type' => 'string',
					"enum" => [
						'0',
						'1',
					],
				],
				'payment_imoje_merchant_id'   => [
					'type' => 'string',
				],
				'payment_imoje_service_id'    => [
					'type' => 'string',
				],
				'payment_imoje_service_key'   => [
					'type' => 'string',
				],
				'payment_imoje_sort_order'    => [
					'type' => 'string',
				],
				'payment_imoje_payment_title' => [
					'type' => 'string',
				],
				'payment_imoje_currencies'    => [
					'type'        => 'array',
					'items'       => [
						'type' => 'string',
						'enum' => array_values(\Imoje\Payment\Util::getSupportedCurrencies()),
					],
					'minItems'    => 1,
					'uniqueItems' => true,
				],
				'payment_imoje_order_history' => [
					'type' => 'string',
					"enum" => [
						'0',
						'1',
					],
				],
				'payment_imoje_send_cart'     => [
					'type' => 'string',
					"enum" => [
						'0',
						'1',
					],
				],
			],

			'required'             => [
				'payment_imoje_status',
				'payment_imoje_debug_mode',
				'payment_imoje_sandbox',
				'payment_imoje_merchant_id',
				'payment_imoje_service_id',
				'payment_imoje_service_key',
				'payment_imoje_sort_order',
				'payment_imoje_payment_title',
				'payment_imoje_currencies',
				'payment_imoje_order_history',
				'payment_imoje_send_cart',
			],
			'additionalProperties' => false,
		];

		$data = json_decode(json_encode($request));

		$validator = new \JsonSchema\Validator();
		$validator->validate($data, $schema);

		$this->error = [];

		foreach($validator->getErrors() as $key => $value) {

			if($value['constraint'] === 'additionalProp') {
				$this->error['additional_properties'] = true;
			} else {
				$this->error[$value['property']] = true;
			}
		}

		return empty($this->error);
	}
}
